/*
   This demo opens /dev/comedi0 and looks for an analog input
   subdevice.  If it finds one, it measures one sample on each
   channel for each input range.  The value NaN indicates that
   the measurement was out of range.
 */

#include <stdio.h>
#include <comedilib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <getopt.h>
#include <ctype.h>

char *filename="/dev/comedi0";


int main(int argc, char *argv[])
{
	comedi_t *it;
	int n_subdevs,subd;
	int n_chans,chan;
	int rangetype,range;
	int maxdata;
	lsampl_t data;
	double voltage;


	it=comedi_open(filename);
	if(!it){
		printf("could not open %s\n",filename);
		exit(0);
	}

	n_subdevs=comedi_get_n_subdevices(it);
	for(subd=0;subd<n_subdevs;subd++){
		if(comedi_get_subdevice_type(it,subd)==COMEDI_SUBD_AI){
			goto subd_found;
		}
	}
	printf("no analog input subdevice found\n");
	exit(0);

subd_found:

	n_chans=comedi_get_n_channels(it,subd);
	for(chan=0;chan<n_chans;chan++){
		printf("%d: ",chan);
		rangetype=comedi_get_rangetype(it,subd,chan);
		maxdata=comedi_get_maxdata(it,subd,chan);
		for(range=0;range<RANGE_LENGTH(rangetype);range++){
			comedi_data_read(it,subd,chan,range,AREF_GROUND,&data);
			voltage=comedi_to_phys(data,comedi_get_range(it,subd,chan,range),maxdata);
			printf("%g ",voltage);
		}
		printf("\n");
	}
	
	exit(0);
}

